<?php
/**
 * @package    FlexPages
 * @copyright  2015 FlexCoders Ltd
 * @license    MIT
 * @link       https://github.com/FlexCoders/FlexPages
 * @author     FlexCoders Ltd
 */

namespace FlexCoders\FlexPages;

use Fuel\Common\Arr;
use Closure;

/**
 * Responsible for building navigation interfaces such as a content menu and
 * breadcrumb trails.
 */
class Navigation
{

	/**
	 * Contains a tree as generated by the TreeCreator class
	 * @var array
	 */
	protected $tree = [];

	/**
	 * Base path of our MD files.
	 * @var string
	 */
	protected $path;

	/**
	 * The current URI that is being accessed
	 * @var string
	 */
	protected $uri;

	/**
	 * Closure to pre-process the filename found before attempting to translate it
	 * @var Closure
	 */
	protected $preProcessor;

	/**
	 * @param string $tree  Current docs tree
	 * @param string $uri  Current request URI
	 * @param string $path Base path for MD files
	 */
	public function __construct($tree, $uri, $path)
	{
		$this->tree = $tree;
		$this->uri = $uri;
		$this->path = $path;
	}

	/**
	 * Optional closure to pre-process filenames before attempting to
	 * translate them. This could include things like stripping sequence
	 * numbers or file extensions.
	 *
	 * @param Closure $preProcessor
	 *
	 * @return self
	 */
	public function setPreProcessor(Closure $preProcessor)
	{
		$this->preProcessor = $preProcessor;

		return $this;
	}

	/**
	 * @return array
	 */
	public function getBreadcrumbList()
	{
		$crumbs = [];

		$tree = $this->tree;

		// Work through our parts and see if there is a translation for them
		foreach (explode('/', $this->uri) as $crumb)
		{
			// do a standard lookup first
			$lang = Arr::get($tree, $crumb, null);

			// if it's an array, it's a folder, so fetch it's title
			if (is_array($lang))
			{
				$crumbs[$crumb] = Arr::get($lang, '__title', $crumb);
				$tree = $lang;
			}

			// if it's a string, it's a file entry, just use it
			elseif ($lang)
			{
				$crumbs[$crumb] = $lang;
			}

			// no hit, it might be a preprocessed entry, do a manhual lookup
			else
			{
				// start with an untranslated default
				$crumbs[$crumb] = $crumb;

				// loop over the translations to find a match
				foreach ($tree as $name => $lang)
				{
					// do we need preprocessing of the filename found?
					if ($this->preProcessor instanceOf Closure)
					{
						$lookup = $this->preProcessor->__invoke($name);
					}
					else
					{
						$lookup = $name;
					}

					if ($lookup == $crumb)
					{
						$crumbs[$crumb] = $lang;
						break;
					}
				}
			}
		}

		return $crumbs;
	}
}
